/*****************************************************
This program was produced by the
CodeWizardAVR V2.04.4a Advanced
Automatic Program Generator

Project : 
Version : 
Date    : 12.11.2010


Chip type               : ATmega164PV
Program type            : Application
AVR Core Clock frequency: 8,000000 MHz
Memory model            : Small
External RAM size       : 0
Data Stack size         : 256
*****************************************************/

#include "config.h"

#include <mega164.h>

// 1 Wire Bus functions
#asm
   .equ __w1_port=0x08 ;PORTC
   .equ __w1_bit=3
#endasm
#include <1wire.h>


// DS1820 Temperature Sensor functions
#include <ds1820.h>

#ifdef  USE_DS18B20
#include <ds18b20.h>
#endif

// Alphanumeric LCD Module functions
#asm
   .equ __lcd_port=0x08 ;PORTC
#endasm
#include <lcd.h>

#include <delay.h>
#include <string.h>
#include <stdio.h>
#include <sleep.h>

#include "avr_types.h"

// Declare global variables
#define __DEF_GLOBAL_VARS__ 1
#include "global_vars.h"
#undef __DEF_GLOBAL_VARS__

#include "timer_interrupt.h"

// Timer 0 overflow interrupt service routine
interrupt [TIM0_OVF] void timer0_ovf_isr(void)
{
	// Reinitialize Timer 0 value
	TCNT0=0x64;
	timer_interrupt();
}


// -----------------------------------------------------------------------------
void clean_line(uint8_t nr)
{
	lcd_gotoxy (0, nr);
	lcd_puts("                ");
	lcd_gotoxy (0, nr);
}

// -----------------------------------------------------------------------------
void error_loop(void)
{
#ifdef TURN_ON_IF_DS1820_FAILURE
	PORTD &= (~ 0b10000000 ); // On
#endif

	clean_line(0);
	lcd_puts("ERROR");
	clean_line(1);
	lcd_puts("DS1820 failure");
	delay_ms(10000);
#asm("jmp 0")
}

// -----------------------------------------------------------------------------
void measure_temp(void)
{
	int16_t my_temps[MAX_DS1820];
	uint8_t tc, i, fail=0;

	if (!flg_ds1820_enabled)
		return;

	for (i=0; i< MAX_DS1820; i++)
	{
#asm ("wdr")
		tc = timer0_cntr +1;
		while (tc != timer0_cntr) {}

#ifdef USE_DS18S20
		my_temps[i] = ds1820_temperature_10(&ds1820_rom_codes[i][0] * 10 );
#endif

#ifdef USE_DS18B20
		my_temps[i] = ds18b20_temperature (&ds1820_rom_codes[i][0]) * 100;
#endif


		if (my_temps[i] < (TEMP_MIN*100) || my_temps[i] > (TEMP_MAX*100) )
		{
			my_temps[i] = TEMP_INVALID;
			fail=1;
		}
	}


	if (fail)
	{
		if (temp_failures)
			temp_failures--;
	}
	else
	{
		temp_failures = TEMP_FAILURES_MAX;
		temps[0]=my_temps[0];
	}


	if (! temp_failures)
	{
		flg_ds1820_enabled=0;
		temps[0]=TEMP_INVALID;
	}	
}

// -----------------------------------------------------------------------------
static flash const char animation[] = "|/-\xcd"; //a4";

void animate(void)
{
	snprintf (temp_buf, sizeof(temp_buf)-1, "%c", animation[ (anim_cntr++ & 0b00000011) ] );
	lcd_gotoxy (15, 0);
	lcd_puts(temp_buf);
}


// -----------------------------------------------------------------------------
void display_temp(void)
{
	int16_t temp = temps[0];
	clean_line(0);
	lcd_puts("T= ");
	if (temp<0)
	{
		temp = -(temp);
		lcd_puts("-");
	}
	else if (temp > 0) 
	{
		lcd_puts("+");
	}
	else  
	{
		lcd_puts(" ");
	}

		
	snprintf (temp_buf, sizeof(temp_buf)-1, "%2d,%02.2d", temp/100, temp % 100 /* temps[0]/10*/ );
	lcd_puts(temp_buf);


	//lcd_puts(" \xb0C");
	lcd_puts("\xdfC");
}

// -----------------------------------------------------------------------------
uint8_t getkey (void)
{
	uint8_t key = last_key;
	last_key=KEY_NOKEY;
	return key;
}

void print_setup_temp (int16_t temp)
{
	clean_line(1);
	snprintf (temp_buf, sizeof(temp_buf)-1, "%2d", temp )  ;
	lcd_gotoxy (0, 1);
	lcd_puts(temp_buf);
	lcd_puts("\xdfC");

}

void setup (void)
{
	uint8_t ch, need_delay=1;

	clean_line(0);
	lcd_puts("T min (on)");

	timers[TIMER_MENU] = TIMER_MENU_DFL;

	while (1)
	{
#asm("wdr")

		if (! timers[TIMER_MENU])
                        return;

	WITH_REDRAW1:

		clean_line(1);
		snprintf (temp_buf, sizeof(temp_buf)-1, "+ %2d\xdfC  ", pipe_temp_min);
		lcd_gotoxy (0, 1);
		lcd_puts(temp_buf);

		if (need_delay)
		{
			need_delay=0;
			delay_ms(2000);
		}

		while (1)
		{
#asm("wdr")
			if (! timers[TIMER_MENU])
				return;
			

			ch = getkey();

			if (ch != KEY_NOKEY)
			{
				if (ch == KEY_1)
					break;

				switch (ch)
				{
				case KEY_3:
					if (pipe_temp_min < PIPE_TEMP_MAX)
						pipe_temp_min++;
					break;
					
				case KEY_2:
					if (pipe_temp_min > PIPE_TEMP_MIN)
						pipe_temp_min--;
					break;

				}
				timers[TIMER_MENU] = TIMER_MENU_DFL;
				goto WITH_REDRAW1;
			}
		}


		clean_line(0);
		lcd_puts("T max (off)");

		timers[TIMER_MENU] = TIMER_MENU_DFL;

	WITH_REDRAW2:
		//print_setup_temp(pipe_temp_max);

		if (pipe_temp_max <= pipe_temp_min)
			pipe_temp_max = pipe_temp_min + 1;

		clean_line(1);
		snprintf (temp_buf, sizeof(temp_buf)-1, "+ %2d\xdfC  ", pipe_temp_max);
		lcd_gotoxy (0, 1);
		lcd_puts(temp_buf);
		//lcd_puts(" C");



		while (1)
		{
#asm("wdr")

			if (! timers[TIMER_MENU])
				return;
			
			ch = getkey();
			
			if (ch != KEY_NOKEY)
			{
				if (ch == KEY_1)
					break;

				switch (ch)
				{
				case KEY_3:
					if (pipe_temp_max < PIPE_TEMP_MAX)
						pipe_temp_max++;
					break;
					
				case KEY_2:
					if (pipe_temp_max > PIPE_TEMP_MIN)
						pipe_temp_max--;
					break;

				}
				timers[TIMER_MENU] = TIMER_MENU_DFL;
				goto WITH_REDRAW2;
			}
		}
		
		break;
	}

	if (pipe_temp_max <= pipe_temp_min)
		pipe_temp_max = pipe_temp_min + 1;


}

// -----------------------------------------------------------------------------
void main(void)
{
	// Declare your local variables here
	uint8_t i, ch;


#include "main_initialization.inc"

	// Determine the number of DS1820 devices
	// connected to the 1 Wire bus
	ds1820_devices=w1_search(0xf0,ds1820_rom_codes);

	// LCD module initialization
	lcd_init(16);

	_lcd_ready();
	lcd_gotoxy (0, 0);
	lcd_puts(LCD_LOGO);
	lcd_gotoxy (0, 1);
	lcd_puts(LCD_COPY);

	delay_ms(DELAY_LOGO);
	lcd_clear();

	// Watchdog Timer initialization
	// Watchdog Timer Prescaler: OSC/1024k
	// Watchdog Timer interrupt: Off
#pragma optsize-
#asm("wdr")
	WDTCSR=0x39;
	WDTCSR=0x29;
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif

// Global enable interrupts
#asm("sei")
	    timer0_cntr = 0;
	    last_key = KEY_NOKEY;

	    for (i=0; i< TIMERS_MAX; i++)
		    timers[i]=0;


	    temp_failures = TEMP_FAILURES_MAX;
	    temps[0] = TEMP_INVALID;

	    flg_ds1820_enabled = 1;

	    kbd_inactiv_timer = 0;

	    if (pipe_temp_min < PIPE_TEMP_MIN || pipe_temp_min > PIPE_TEMP_MAX)
		pipe_temp_min = PIPE_TEMP_MIN_DFL;

	    if (pipe_temp_max < PIPE_TEMP_MIN || pipe_temp_max > PIPE_TEMP_MAX)
		    pipe_temp_max = PIPE_TEMP_MAX_DFL ;


	    
	    if (ds1820_devices != MAX_DS1820)
		    error_loop();


#ifdef USE_DS18B20
	for (i=0; i< MAX_DS1820; i++)
	{
		ds18b20_init (&ds1820_rom_codes[i][0], -55, 125, DS18B20_12BIT_RES);
	}
#endif

	    measure_temp();
	    display_temp();

	while (1)
	{
      // Place your code here
#asm("wdr")
		keyboard_repeats_set= KEYBOARD_REPEATS_SET;

		clean_line(1);
		lcd_puts("Ready...");

		lcd_gotoxy (11, 1);

		if (PORTD & 0b10000000)
			lcd_puts("[Off]");
		else
			lcd_puts("[On ]");
		

		animate();


		if (!timers[TIMER_TEMP])
		{
			timers[TIMER_TEMP]=TIMER_TEMP_DFL;
			measure_temp();
			display_temp();
		}



		if (flg_ds1820_enabled)
		{
			if ( temps[0] <= (pipe_temp_min * 100) )
				if (PORTD & 0b10000000) // Off
					PORTD &= (~ 0b10000000 ); // On

			if (temps[0] >= (pipe_temp_max * 100) )
				if (!(PORTD & 0b10000000)) // On
					PORTD |= 0b10000000; // Off
		}



		ch = getkey();

		if (ch != KEY_NOKEY)
		{

			keyboard_repeats_set= KEYBOARD_REPEATS;

			/*
			snprintf (temp_buf, sizeof(temp_buf)-1, "0x%x", ch )  ;
			lcd_gotoxy (12, 1);
			lcd_puts(temp_buf);
			delay_ms(500);
			*/

			switch (ch)
			{
			case KEY_1:
				setup();
				break;



			case KEY_2:
			case KEY_3:

				timers[TIMER_BND] = TIMER_BND_DFL;	

				clean_line(1);
				lcd_puts("Temp range");

				clean_line(0);

				snprintf (temp_buf,
					  sizeof(temp_buf)-1,
					  "+%d\xdfC...+%d\xdfC",
					  pipe_temp_min,
					  pipe_temp_max );
				lcd_puts(temp_buf);


				while (timers[TIMER_BND]) 
				{
#asm("wdr")
					animate();
					delay_ms(200);
				}
				break;

			}


			delay_ms(500);
			display_temp();
			timers[TIMER_TEMP]=TIMER_TEMP_DFL;
			timers[TIMER_BND] = TIMER_BND_DFL;	
		}


		delay_ms(200);


		if (!flg_ds1820_enabled)
			error_loop();



	};
}

// __END__
